//
//  ViewController.m
//  StreamInputTtsDemo，演示完整的流式语音合成IOS SDK使用方法。
//
//  Created by lengjiayi on 2024/4/22.
//

#import "ViewController.h"
#import "audio/NLSPlayAudio.h"
#import "nuisdk.framework/Headers/StreamInputTts.h"

//测试用文本，每隔一秒发送一行文本。
NSString *const demoText =
    @"流式文本语音合成SDK，\n可以将输入的文本\n合成为语音二进制数据，\n相比于非"
    @"流式语音合成，\n流式合成的优势在于实时性\n更强。用户在输入文本的同时\n可"
    @"以听到接近同步的语音输出，\n极大地提升了交互体验，\n减少了用户等待时间。"
    @"\n适用于调用大规模\n语言模型（LLM），以\n流式输入文本的方式\n进行语音合成"
    @"的场景。";

@interface ViewController () <NlsPlayerDelegate,
                              UITextFieldDelegate,
                              StreamInputTtsDelegate>

@property(nonatomic, strong) StreamInputTts *streamInputTtsSdk;
@property(nonatomic, strong) NLSPlayAudio *voicePlayer;
@property(nonatomic, strong) NSThread *sendThread;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    // 创建音频播放器，默认使用24000采样率
    _voicePlayer = [[NLSPlayAudio alloc] init];
    [_voicePlayer setsamplerate:24000];
    _voicePlayer.delegate = self;
    // 获得流式TTS实例，并配置delegate
    _streamInputTtsSdk = [StreamInputTts get_instance];
    _streamInputTtsSdk.delegate = self;
    [self setupUIObjects];
}

- (void)dealloc {
    TLog(@"%s", __FUNCTION__);
    [_voicePlayer cleanup];
}

/**
 * 获取入会所需鉴权参数,
 * 代码中"your-appkey"以及"your-token"字符串变量需要您自行填入，url默认使用北京服务
 */
- (NSString *)genTicket {
    NSDictionary *jsonDict = @{
        @"appkey" : @"your-appkey",
        @"token" : @"your-token",
        @"url" : @"wss://nls-gateway-cn-beijing.aliyuncs.com/ws/v1",
        @"complete_waiting_ms" : @"10000",
    };
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:jsonDict
                                                       options:0
                                                         error:&error];
    NSString *ticket = [[NSString alloc] initWithData:jsonData
                                             encoding:NSUTF8StringEncoding];
    return ticket;
}

/**
 * 语音合成相关的参数，默认使用24000 采样率。
 */
- (NSString *)genParameters {
    NSDictionary *jsonDict = @{
        @"voice" : @"longxiaochun",
        @"format" : @"pcm",
        @"sample_rate" : @"24000",
        @"volume" : @"50",
        @"speech_rate" : @"0",
        @"pitch_rate" : @"0",
        @"enable_subtitle" : @"0",
        @"session_id" : @"",
    };
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:jsonDict
                                                       options:0
                                                         error:&error];
    NSString *parameters = [[NSString alloc] initWithData:jsonData
                                                 encoding:NSUTF8StringEncoding];
    return parameters;
}

/**
 * 配置前端控件
 */
- (void)setupUIObjects {
    // titleLabel为app标题
    self.titleLabel = [[UILabel alloc]
        initWithFrame:CGRectMake(20, 80, self.view.bounds.size.width - 40, 50)];
    self.titleLabel.text = @"流式文本语音合成Demo";
    self.titleLabel.font = [UIFont systemFontOfSize:24];
    [self.view addSubview:self.titleLabel];

    // logTextView用于输出log信息
    self.logTextView = [[UITextView alloc]
        initWithFrame:CGRectMake(20, CGRectGetMaxY(self.titleLabel.frame) + 20,
                                 self.view.bounds.size.width - 40, 200)];
    self.logTextView.alwaysBounceVertical = YES;
    self.logTextView.scrollEnabled = YES;
    self.logTextView.editable = NO;  // 设置为只读以用作日志显示
    self.logTextView.textContainer.lineBreakMode = NSLineBreakByTruncatingTail;
    self.logTextView.text = [self.logTextView.text
        stringByAppendingString:@"输出日志（详细日志请参考Console Log）"];
    self.logTextView.layer.borderColor = [[UIColor blackColor] CGColor];
    self.logTextView.layer.borderWidth = 1.0;
    [self.view addSubview:self.logTextView];

    // inputTextField用于显示测试文本
    self.inputTextField = [[UITextView alloc]
        initWithFrame:CGRectMake(20, CGRectGetMaxY(self.logTextView.frame) + 20,
                                 self.view.bounds.size.width - 40, 200)];
    self.inputTextField.editable = YES;  // 可以输入为您希望合成的内容
    self.inputTextField.text = demoText;
    self.inputTextField.borderStyle = UITextBorderStyleRoundedRect;
    self.inputTextField.layer.borderColor = [[UIColor blackColor] CGColor];
    self.inputTextField.layer.borderWidth = 1.0;
    [self.view addSubview:self.inputTextField];
    self.inputTextField.delegate = self;

    // 开始按钮，点击后开始执行一次流式tts任务
    CGFloat buttonWidth = 100;
    CGFloat buttonHeight = 50.0;
    CGFloat padding = 10.0;
    CGFloat startY = CGRectGetMaxY(self.inputTextField.frame) + 20;
    CGFloat startX =
        ([[UIScreen mainScreen] bounds].size.width - buttonWidth) / 2;
    self.buttonStart = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.buttonStart setTitle:[NSString stringWithFormat:@"开始"]
                      forState:UIControlStateNormal];
    [self.buttonStart
        setFrame:CGRectMake(startX, startY, buttonWidth, buttonHeight)];
    [self.buttonStart setFont:[UIFont systemFontOfSize:20]];
    self.buttonStart.layer.borderWidth = 1.0;
    [self.buttonStart addTarget:self
                         action:@selector(startPressed:)
               forControlEvents:UIControlEventTouchUpInside];
    self.buttonStart.tag = 1;
    [self.view addSubview:self.buttonStart];
}
/**
 * 同时在console和logTextView打印出log信息
 */
- (void)logFormattedString:(NSString *)format, ... {
    va_list args;
    va_start(args, format);
    NSString *formattedString = [[NSString alloc] initWithFormat:format
                                                       arguments:args];
    NSLog(@"%@", formattedString);
    dispatch_async(dispatch_get_main_queue(), ^{
      self.logTextView.text =
          [self.logTextView.text stringByAppendingString:formattedString];
      NSRange bottom = NSMakeRange(self.logTextView.text.length - 1, 1);
      [self.logTextView scrollRangeToVisible:bottom];
    });
}

/**
 * 执行一次流式语音合成任务
 */
- (void)doStreamInputTts:(NSString *)inputContents {
    // 获得鉴权信息和语音合成参数
    NSString *ticket = [self genTicket];
    NSString *parameters = [self genParameters];
    // 获得流式TTS实例，并配置delegate
    _streamInputTtsSdk = [StreamInputTts get_instance];
    _streamInputTtsSdk.delegate = self;
    // 建立连接并开始语音合成任务
    int ret = [_streamInputTtsSdk startStreamInputTts:[ticket UTF8String]
                                           parameters:[parameters UTF8String]
                                            sessionId:nil
                                             logLevel:0
                                              saveLog:NO];

    NSArray<NSString *> *lines = [inputContents
        componentsSeparatedByCharactersInSet:[NSCharacterSet
                                                 newlineCharacterSet]];
    unsigned long line_num = [lines count];
    while (line_num > 0) {
        // 取第一行文本
        NSString *oneLine = [lines firstObject];
        [self logFormattedString:@"\n发送文本：%@", oneLine];
        // 发送流式文本（可以多次发送），如果有响应，则执行回调函数
        ret = [_streamInputTtsSdk sendStreamInputTts:[oneLine UTF8String]];
        // 更新文本内容，去除第一行文本
        NSString *updatedText = @"";
        if (line_num > 1) {
            updatedText =
                [[lines subarrayWithRange:NSMakeRange(1, [lines count] - 1)]
                    componentsJoinedByString:@"\n"];
            dispatch_async(dispatch_get_main_queue(), ^{
              self->_inputTextField.text = updatedText;
            });
        } else {
            dispatch_async(dispatch_get_main_queue(), ^{
              self->_inputTextField.text = updatedText;
            });
            break;
        }
        lines = [updatedText
            componentsSeparatedByCharactersInSet:[NSCharacterSet
                                                     newlineCharacterSet]];
        line_num = [lines count];
        // 模拟LLM合成的文本，两次发送间间隔一秒
        [NSThread sleepForTimeInterval:1];
    }
    // 停止发送文本，阻塞至所有音频返回，并结束本次流式语音合成
    ret = [_streamInputTtsSdk stopStreamInputTts];
}

/**
 * 按下“开始”按钮执行此函数
 */
- (void)startPressed:(UIButton *)sender {
    // 音频播放器开始
    [_voicePlayer stop];
    [_voicePlayer play];
    // 由于需要开启线程播放，因此按钮按下后禁用，本DEMO只做一次流式语音合成任务
    [sender setEnabled:NO];
    // 设置待合成文本
    self.inputTextField.text = demoText;
    NSString *inputContents = _inputTextField.text;
    // 创建新线程，并完成一次流式语音合成任务
    NSThread *thread =
        [[NSThread alloc] initWithTarget:self
                                selector:@selector(doStreamInputTts:)
                                  object:inputContents];
    [thread start];
}

#pragma tts callback

/**
 * 事件回调，打印出服务返回的完整信息，可以根据需要处理对应的事件类型
 */
- (void)onStreamInputTtsEventCallback:(StreamInputTtsCallbackEvent)event
                               taskId:(char *)taskid
                            sessionId:(char *)sessionId
                             ret_code:(int)ret_code
                            error_msg:(char *)error_msg
                            timestamp:(char *)timestamp
                         all_response:(char *)all_response {
    //    [self logFormattedString:@"\n[事件回调] (event_code : %d), %s", event,
    //    all_response];
    NSLog(@"\n[事件回调] (event_code : %d), %s", event, all_response);
    // 可以处理各类型的事件
    switch (event) {
        case TTS_EVENT_SYNTHESIS_STARTED:
            // TODO: 处理SynthesisStarted指令
            break;
        case TTS_EVENT_SENTENCE_BEGIN:
            // TODO: 处理SentenceBegin指令
            break;
        case TTS_EVENT_SENTENCE_SYNTHESIS:
            // TODO: 处理SentenceSynthesis指令
            break;
        case TTS_EVENT_SENTENCE_END:
            // TODO: 处理SynthesisEnd指令
            break;
        case TTS_EVENT_SYNTHESIS_COMPLETE:
            // TODO: 处理SynthesisComplete指令
            break;
        case TTS_EVENT_TASK_FAILED:
            // TODO: 处理TaskFailed指令
            break;
        default:
            break;
    }
}

/**
 * 音频回调函数，将收到的音频帧写入到音频播放器中
 */
- (void)onStreamInputTtsDataCallback:(char *)buffer len:(int)len {
    NSLog(@"\n[收到音频] %d 字节", len);
    if (len > 0) {
        [_voicePlayer write:(char *)buffer Length:(unsigned int)len];
    }
}

@end
